@@ -1,8 +1,6 @@
 import os
 from yggdrasil import doctools
 from yggdrasil.metaschema.datatypes.MetaschemaType import MetaschemaType
-from yggdrasil.metaschema.datatypes.tests.test_MetaschemaType import (
-    TestMetaschemaType)
 from yggdrasil.metaschema.properties.MetaschemaProperty import MetaschemaProperty
 from yggdrasil.drivers.ModelDriver import ModelDriver
 from yggdrasil.drivers.InterpretedModelDriver import InterpretedModelDriver
@@ -27,3 +25,5 @@ for c in class_list:
 doctools.write_datatype_table(fname_dir=tables_dir, verbose=False)
 doctools.write_datatype_mapping_table(fname_dir=tables_dir, verbose=False)
 doctools.write_comm_devnotes_table(fname_dir=tables_dir, verbose=False)
+doctools.write_interface_mapping_table(fname_dir=tables_dir, verbose=False,
+                                       style='list')
