@@ -20,7 +20,6 @@ _on_appveyor = bool(os.environ.get('APPVEYOR_BUILD_FOLDER', False))
 _on_ci = (_on_gha or _on_travis or _on_appveyor)
 _utils_dir = os.path.dirname(os.path.abspath(__file__))
 _pkg_dir = os.path.dirname(_utils_dir)
-BUILDDOCS = (os.environ.get('BUILDDOCS', '0') == '1')
 CONDA_ENV = os.environ.get('CONDA_DEFAULT_ENV', None)
 CONDA_PREFIX = os.environ.get('CONDA_PREFIX', None)
 CONDA_INDEX = None
@@ -106,34 +105,50 @@ def call_script(lines, force_bash=False):
     """
     if not lines:
         return
-    if _is_win and (not force_bash):  # pragma: windows
-        script_ext = '.bat'
-        error_check = 'if %errorlevel% neq 0 exit /b %errorlevel%'
-        for i in range(len(lines), 0, -1):
-            lines.insert(i, error_check)
-    else:
-        script_ext = '.sh'
-        if lines[0] != '#!/bin/bash':
-            lines.insert(0, '#!/bin/bash')
-        error_check = 'set -e'
-        if error_check not in lines:
-            lines.insert(1, error_check)
-    fname = 'ci_script_%s%s' % (str(uuid.uuid4()), script_ext)
-    try:
-        pprint.pprint(lines)
-        with open(fname, 'w') as fd:
-            fd.write('\n'.join(lines))
-            
-        call_kws = {}
-        if _is_win:  # pragma: windows
-            call_cmd = [fname]
+    # Split lines that should be allowed to fail
+    line_sets = []
+    idx = 0
+    for i, line in enumerate(lines):
+        if line.endswith('# [ALLOW FAIL]'):
+            line_sets.append(lines[idx:i])
+            line_sets.append([lines[i]])
+            idx = i + 1
+    line_sets.append(lines[idx:])
+    for lines in line_sets:
+        allow_failure = lines[0].endswith('# [ALLOW FAIL]')
+        if allow_failure:
+            lines = [lines[0].split('#')[0].strip()]
+        if _is_win and (not force_bash):  # pragma: windows
+            script_ext = '.bat'
+            error_check = 'if %errorlevel% neq 0 exit /b %errorlevel%'
+            for i in range(len(lines), 0, -1):
+                lines.insert(i, error_check)
         else:
-            call_cmd = ['./%s' % fname]
-            os.chmod(fname, 0o755)
-        subprocess.check_call(call_cmd, **call_kws)
-    finally:
-        if os.path.isfile(fname):
-            os.remove(fname)
+            script_ext = '.sh'
+            if lines[0] != '#!/bin/bash':
+                lines.insert(0, '#!/bin/bash')
+            error_check = 'set -e'
+            if error_check not in lines:
+                lines.insert(1, error_check)
+        fname = 'ci_script_%s%s' % (str(uuid.uuid4()), script_ext)
+        try:
+            pprint.pprint(lines)
+            with open(fname, 'w') as fd:
+                fd.write('\n'.join(lines))
+                
+            call_kws = {}
+            if _is_win:  # pragma: windows
+                call_cmd = [os.environ['COMSPEC'], '/c', 'call', fname]
+            else:
+                call_cmd = ['./%s' % fname]
+                os.chmod(fname, 0o755)
+            subprocess.check_call(call_cmd, **call_kws)
+        except subprocess.CalledProcessError:
+            if not allow_failure:
+                raise
+        finally:
+            if os.path.isfile(fname):
+                os.remove(fname)
 
 
 def conda_env_exists(name):
@@ -216,6 +231,9 @@ def get_install_opts(old=None):
             'trimesh': (os.environ.get('INSTALLTRIMESH', '0') == '1'),
             'pygments': (os.environ.get('INSTALLPYGMENTS', '0') == '1'),
             'omp': (os.environ.get('INSTALLOMP', '0') == '1'),
+            'docs': (os.environ.get('BUILDDOCS', '0') == '1'),
+            'no_sudo': False,
+            'mpi': (os.environ.get('INSTALLMPI', '0') == '1'),
         }
         if not _is_win:
             new['c'] = True  # c compiler usually installed by default
@@ -232,6 +250,9 @@ def get_install_opts(old=None):
             'trimesh': True,
             'pygments': True,
             'omp': False,
+            'docs': False,
+            'no_sudo': False,
+            'mpi': False,
         }
     if _is_win:
         new['os'] = 'win'
@@ -247,6 +268,8 @@ def get_install_opts(old=None):
         out.setdefault(k, v)
     if not out['c']:
         out.update(fortran=False, zmq=False)
+    if out['docs']:
+        out['R'] = True  # Allow roxygen
     return out
 
 
@@ -260,6 +283,11 @@ def add_install_opts_args(parser):
     for k, v in install_opts.items():
         if k in ['os']:
             continue
+        elif k == 'no_sudo':
+            parser.add_argument(
+                '--no-sudo', action='store_true',
+                help="Don't use sudo during installation.")
+            continue
         if v:
             parser.add_argument(
                 '--dont-install-%s' % k, action='store_true',
@@ -575,6 +603,18 @@ def itemize_deps(method, for_development=False,
             out['os'] += ['libomp', 'llvm']
         elif install_opts['os'] == 'win':
             pass
+    if install_opts['mpi'] and (not fallback_to_conda):
+        if install_opts['os'] == 'linux':
+            out['os'] += ['openmpi-bin', 'libopenmpi-dev']
+        elif install_opts['os'] == 'osx':
+            out['os'].append('openmpi')
+        elif install_opts['os'] == 'win':
+            pass
+    elif ((install_opts['os'] == 'win') and install_opts['mpi']
+          and (method == 'conda')):
+        # Force mpi4py to be installed last on Windows to avoid
+        # conflicts
+        out['skip'].append('mpi4py')
     if install_opts['fortran'] and (not fallback_to_conda):
         # Fortran is not installed via conda on linux/macos
         if install_opts['os'] == 'linux':
@@ -614,10 +654,17 @@ def itemize_deps(method, for_development=False,
         else:
             raise NotImplementedError("Could not determine "
                                       "ZeroMQ installation method.")
-    if include_doc_deps or BUILDDOCS:
+    if include_doc_deps or install_opts['docs']:
         out['requirements'].append('requirements_documentation.txt')
         if not fallback_to_conda:
             out['os'].append("doxygen")
+            if install_opts['os'] == 'linux':
+                out['os'] += ["pandoc"]
+            elif install_opts['os'] == 'osx':
+                out['os'] += ["pandoc-citeproc", "Caskroom/cask/mactex"]
+            else:
+                NotImplementedError("Could not determine "
+                                    "pandoc installation method.")
     if install_opts['sbml'] and fallback_to_conda:
         # Until the sbml package is updated to allow numpy != 1.19.3,
         # sbml will need to be installed separately without deps in order
@@ -729,11 +776,18 @@ def install_deps(method, return_commands=False, verbose=False,
         # TODO: Test split installation where r-base is installed from
         # conda and the R dependencies are installed from CRAN?
         if _is_linux:
-            cmds += [
-                ("sudo add-apt-repository 'deb https://cloud"
-                 ".r-project.org/bin/linux/ubuntu xenial-cran35/'"),
-                ("sudo apt-key adv --keyserver keyserver.ubuntu.com "
-                 "--recv-keys E298A3A825C0D65DFD57CBB651716619E084DAB9")]
+            if install_opts['no_sudo']:
+                cmds += [
+                    ("add-apt-repository 'deb https://cloud"
+                     ".r-project.org/bin/linux/ubuntu xenial-cran35/'"),
+                    ("apt-key adv --keyserver keyserver.ubuntu.com "
+                     "--recv-keys E298A3A825C0D65DFD57CBB651716619E084DAB9")]
+            else:
+                cmds += [
+                    ("sudo add-apt-repository 'deb https://cloud"
+                     ".r-project.org/bin/linux/ubuntu xenial-cran35/'"),
+                    ("sudo apt-key adv --keyserver keyserver.ubuntu.com "
+                     "--recv-keys E298A3A825C0D65DFD57CBB651716619E084DAB9")]
     # if install_opts['zmq'] and (not fallback_to_conda):
     #     cmds.append("echo Installing ZeroMQ...")
     #     if _is_linux:
@@ -760,8 +814,12 @@ def install_deps(method, return_commands=False, verbose=False,
     # Install dependencies using package manager(s)
     if not only_python:
         if pkgs['apt']:
-            cmds += ["sudo apt update"]
-            cmds += ["sudo apt-get install %s" % ' '.join(pkgs['apt'])]
+            if install_opts['no_sudo']:
+                cmds += ["apt update"]
+                cmds += ["apt-get install %s" % ' '.join(pkgs['apt'])]
+            else:
+                cmds += ["sudo apt update"]
+                cmds += ["sudo apt-get install %s" % ' '.join(pkgs['apt'])]
         if pkgs['brew']:
             if 'gcc' in pkgs['brew']:
                 cmds += ["brew reinstall gcc"]
@@ -823,6 +881,18 @@ def install_deps(method, return_commands=False, verbose=False,
             raise RuntimeError("Could not detect conda environment. "
                                "Cannot proceed with a conda deployment "
                                "(required for LPy).")
+    # if _is_win and install_opts['mpi'] and fallback_to_conda:
+    #     if verbose:
+    #         install_flags = '-vvv'
+    #     else:
+    #         install_flags = '-q'
+    #     install_flags = '-vv'
+    #     install_flags += conda_flags
+    #     # This is required as the install script for mpi4py aborts without
+    #     # an error message when called inside a Python subprocess. This seems
+    #     # to occur during cleanup for the installation process as the mpi4py
+    #     # installation is functional. Possibly triggered by the activation script?
+    #     cmds += ["%s install %s mpi4py # [ALLOW FAIL]" % (CONDA_CMD, install_flags)]
     if return_commands:
         return cmds
     cmds = SUMMARY_CMDS + cmds + SUMMARY_CMDS
@@ -934,6 +1004,8 @@ def install_pkg(method, python=None, without_build=False,
             # "%s install %s --update-deps -c %s yggdrasil \"blas=*=openblas\"" % (
             #     CONDA_CMD, install_flags, index_channel)
         ]
+        if _is_win and install_opts['mpi']:
+            cmds[-1] = cmds[-1] + ' mpi4py # [ALLOW FAIL]'
     elif method == 'pip':
         if verbose:
             install_flags = '--verbose'
@@ -967,7 +1039,10 @@ def install_pkg(method, python=None, without_build=False,
     # Follow up if on Unix as R installation may require sudo
     if install_opts['R'] and _is_unix:
         # cmds.append('ygginstall r --sudoR')
-        subprocess.check_call(["ygginstall", "r", "--sudoR"])
+        R_cmd = ["ygginstall", "r"]
+        if not install_opts['no_sudo']:
+            R_cmd.append("--sudoR")
+        subprocess.check_call(R_cmd)
     if method == 'conda':
         src_dir = os.path.join(os.getcwd(),
                                os.path.dirname(os.path.dirname(__file__)))
@@ -989,14 +1064,14 @@ def verify_pkg(install_opts=None):
         install_opts['fortran'] = False
     elif (not install_opts['fortran']) and shutil.which('gfortran'):
         install_opts['fortran'] = True
-    if (not install_opts['R']) and shutil.which('Rscript'):
-        if not (_on_gha and _is_linux):
-            # The installation on GHA-ubuntu machines requires sudo
-            # and so installation will not be complete unless it is
-            # enabled explicitly. This does not seem to be True on
-            # GHA-macos builds and R is not installed by default on
-            # Travis/Appveyor.
-            install_opts['R'] = True
+    # if (not install_opts['R']) and shutil.which('Rscript'):
+    #     if not (_on_gha and _is_linux):
+    #         # The installation on GHA-ubuntu machines requires sudo
+    #         # and so installation will not be complete unless it is
+    #         # enabled explicitly. This does not seem to be True on
+    #         # GHA-macos builds and R is not installed by default on
+    #         # Travis/Appveyor.
+    #         install_opts['R'] = True
     if (not install_opts['c']) and shutil.which('gcc') and (not _is_win):
         install_opts['c'] = True
     src_dir = os.path.join(os.getcwd(),
@@ -1036,6 +1111,12 @@ def verify_pkg(install_opts=None):
             errors.append("Language '%s' should be installed, but is not."
                           % name)
         elif (not flag) and is_lang_installed(name):
+            if name in ['R']:
+                # Allow R to be installed even if the settings is not as
+                # packages may be installed from CRAN, unless there is an
+                # error which can occur when a new release of a dependency
+                # comes out but there are not yet binaries available
+                continue
             errors.append("Language '%s' should NOT be installed, but is."
                           % name)
     for name in ['zmq', 'rmq']:
@@ -1262,7 +1343,9 @@ if __name__ == "__main__":
     if args.operation in ['deps', 'install', 'verify', 'env-yaml']:
         new_opts = {}
         for k, v in install_opts.items():
-            if v and getattr(args, 'dont_install_%s' % k, False):
+            if k == 'no_sudo':
+                new_opts[k] = bool(getattr(args, k, False))
+            elif v and getattr(args, 'dont_install_%s' % k, False):
                 new_opts[k] = False
             elif (not v) and getattr(args, 'install_%s' % k, False):
                 new_opts[k] = True
