@@ -1,3 +1,4 @@
+from yggdrasil import multitasking
 from yggdrasil.communication import CommBase, NoMessages
 from yggdrasil.config import ygg_cfg
 import logging
@@ -6,8 +7,8 @@ try:
     import pika
     _rmq_installed = True
     _pika_version_maj = int(float(pika.__version__.split('.')[0]))
-    if _pika_version_maj >= 1:  # pragma: debug
-        raise ImportError("pika version 1.0 not yet supported.")
+    if _pika_version_maj < 1:  # pragma: debug
+        raise ImportError("pika version <1.0 no longer supported.")
 except ImportError:
     logger.debug("Could not import pika. "
                  + "RabbitMQ support will be disabled.")
@@ -60,7 +61,7 @@ def check_rmq_server(url=None, **kwargs):
     try:
         connection = pika.BlockingConnection(parameters)
         if not connection.is_open:  # pragma: debug
-            raise BaseException("Connection was not openned.")
+            raise BaseException("Connection was not opened.")
         connection.close()
     except BaseException as e:  # pragma: debug
         print("Error when attempting to connect to the RabbitMQ server: "
@@ -109,13 +110,17 @@ class RMQComm(CommBase.CommBase):
                            "``exchange`` is the name of the exchange on the queue "
                            "that should be used, and ``queue`` is the name of "
                            "the queue.")
+    _disconnect_attr = (CommBase.CommBase._disconnect_attr
+                        + ['_opening', '_closing'])
     
     def _init_before_open(self, **kwargs):
         r"""Set null connection and channel."""
+        if not hasattr(self, 'rmq_lock'):
+            self.rmq_lock = multitasking.RLock()
         self.connection = None
         self.channel = None
-        self._is_open = False
-        self._bound = False
+        self._opening = multitasking.ProcessEvent()
+        self._closing = multitasking.ProcessEvent()
         self._server_class = RMQServer
         super(RMQComm, self)._init_before_open(**kwargs)
 
@@ -207,41 +212,29 @@ class RMQComm(CommBase.CommBase):
 
     def bind(self):
         r"""Declare queue to get random new queue."""
-        if self.is_open or self._bound:
+        if self._opening.has_started() or self._closing.has_started():
             return
-        self._bound = True
-        parameters = pika.URLParameters(self.url)
-        self.connection = pika.BlockingConnection(parameters)
-        self.channel = self.connection.channel()
-        self.channel.exchange_declare(exchange=self.exchange,
-                                      auto_delete=True)
-        if self.direction == 'recv' and not self.queue:
-            exclusive = False  # True
-        else:
-            exclusive = False
-        if self.queue.startswith('amq.'):
-            passive = True
-        else:
-            passive = False
-        res = self.channel.queue_declare(queue=self.queue,
-                                         exclusive=exclusive,
-                                         passive=passive,
-                                         auto_delete=True)
-        if not self.queue:
-            self.address += res.method.queue
-        self.channel.queue_bind(exchange=self.exchange,
-                                # routing_key=self.routing_key,
-                                queue=self.queue)
-        self.register_comm(self.address, (self.connection, self.channel))
-        super(RMQComm, self).bind()
-
-    def open(self):
-        r"""Open connection and bind/connect to queue as necessary."""
-        super(RMQComm, self).open()
-        if not self.is_open:
-            self.bind()
-            self._is_open = True
-            self._bound = False
+        self._opening.start()
+        with self.rmq_lock:
+            if self.connection is None:
+                parameters = pika.URLParameters(self.url)
+                self.connection = pika.BlockingConnection(parameters)
+                self.original_queue = self.queue
+            self.channel = self.connection.channel()
+            self.channel.exchange_declare(exchange=self.exchange,
+                                          auto_delete=True)
+            if self.direction == 'send':
+                self.channel.confirm_delivery()
+            res = self.channel.queue_declare(
+                queue=self.queue, exclusive=False, auto_delete=True,
+                passive=self.original_queue.startswith('amq.'))
+            if not self.queue:
+                self.address += res.method.queue
+            self.channel.queue_bind(exchange=self.exchange,
+                                    queue=self.queue)
+            self.register_comm(self.address, (self.connection, self.channel))
+            super(RMQComm, self).bind()
+        self._opening.stop()
 
     def _close(self, linger=False):
         r"""Close the connection.
@@ -251,106 +244,102 @@ class RMQComm(CommBase.CommBase):
                 comm. Defaults to False.
 
         """
-        self._is_open = False
-        self._bound = False
+        if self._closing.has_started():  # pragma: debug
+            return
+        self._closing.start()
         if self.direction == 'recv':
             self.close_queue()
         self.close_channel()
         self.close_connection()
         if not self.is_client:
             self.unregister_comm(self.address)
-        self.connection = None
-        self.channel = None
+        with self.rmq_lock:
+            self.channel = None
+            self.connection = None
         super(RMQComm, self)._close(linger=linger)
+        self._closing.stop()
 
-    def close_queue(self):
+    def close_queue(self, skip_unbind=False):
         r"""Close the queue if the channel exists."""
-        if self.channel and (not self.is_client):
-            try:
-                self.channel.queue_unbind(queue=self.queue,
-                                          exchange=self.exchange)
-                self.channel.queue_delete(queue=self.queue)
-            except (pika.exceptions.ChannelClosed,
-                    pika.exceptions.ConnectionClosed):  # pragma: debug
-                pass
-            except AttributeError:  # pragma: debug
-                if self.channel is not None:
-                    raise
+        with self.rmq_lock:
+            if self.channel and (not self.is_client):
+                try:
+                    if not skip_unbind:
+                        self.channel.queue_unbind(queue=self.queue,
+                                                  exchange=self.exchange)
+                    self.channel.queue_delete(queue=self.queue)
+                except (pika.exceptions.ChannelClosed,
+                        pika.exceptions.ConnectionClosed,
+                        pika.exceptions.ChannelWrongStateError,
+                        pika.exceptions.ConnectionWrongStateError):  # pragma: debug
+                    pass
+                except AttributeError:  # pragma: debug
+                    if self.channel is not None:
+                        raise
 
     def close_channel(self):
         r"""Close the channel if it exists."""
-        if self.channel:
-            try:
+        with self.rmq_lock:
+            if self.channel is not None:
+                self.debug('Closing the channel')
                 self.channel.close()
-            except (pika.exceptions.ChannelClosed,
-                    pika.exceptions.ConnectionClosed,
-                    pika.exceptions.ChannelAlreadyClosing):  # pragma: debug
-                pass
-        self.channel = None
 
     def close_connection(self):
         r"""Close the connection."""
-        if self.connection:
-            try:
+        with self.rmq_lock:
+            if self.connection is not None:
+                self.debug('Closing connection')
                 self.connection.close()
-            except (pika.exceptions.ChannelClosed,
-                    pika.exceptions.ConnectionClosed,
-                    pika.exceptions.ChannelAlreadyClosing):  # pragma: debug
-                pass
-            except AttributeError:  # pragma: debug
-                pass
-        self.connection = None
 
     @property
     def is_open(self):
         r"""bool: True if the connection and channel are open."""
-        # with self._closing_thread.lock:
-        if self.channel is None or self.connection is None:
-            return False
-        if self.connection.is_open:
-            if self.connection.is_closing:  # pragma: debug
+        with self.rmq_lock:
+            if self.channel is None or self.connection is None:
                 return False
-        else:  # pragma: debug
-            return False
-        if self.channel.is_open:
-            if self.channel.is_closing:  # pragma: debug
+            if not self.connection.is_open:
                 return False
-        else:  # pragma: debug
-            return False
-        return self._is_open
+            if not self.channel.is_open:
+                return False
+            return (self._opening.has_stopped()
+                    and (not self._closing.has_started()))
 
     def get_queue_result(self):
         r"""Get the fram from passive queue declare."""
-        res = None
-        if self.is_open:
-            try:
-                res = self.channel.queue_declare(queue=self.queue,
-                                                 auto_delete=True,
-                                                 passive=True)
-            except BlockingIOError:  # pragma: debug
-                self.sleep()
-                res = self.get_queue_result()
-            except (pika.exceptions.ChannelClosed,
-                    pika.exceptions.ConnectionClosed,
-                    AttributeError):  # pragma: debug
-                self._close()
-        return res
+        with self.rmq_lock:
+            res = None
+            if self.is_open:
+                try:
+                    res = self.channel.queue_declare(queue=self.queue,
+                                                     auto_delete=True,
+                                                     passive=True)
+                except pika.exceptions.ChannelClosedByBroker:  # pragma: debug
+                    self._close()
+                # except BlockingIOError:  # pragma: debug
+                #     self.sleep()
+                #     res = self.get_queue_result()
+                # except (pika.exceptions.ChannelClosed,
+                #         pika.exceptions.ConnectionClosed,
+                #         pika.exceptions.ChannelWrongStateError,
+                #         pika.exceptions.ConnectionWrongStateError,
+                #         pika.exceptions.StreamLostError,
+                #         AttributeError):  # pragma: debug
+                #     self._close()
+            return res
         
     @property
     def n_msg_recv(self):
         r"""int: Number of messages in the queue."""
-        return self.n_msg_send
+        out = 0
+        res = self.get_queue_result()
+        if res is not None:
+            out = res.method.message_count
+        return out
 
     @property
     def n_msg_send(self):
         r"""int: Number of messages in the queue."""
-        out = 0
-        # with self._closing_thread.lock:
-        if self.is_open:
-            res = self.get_queue_result()
-            if res is not None:
-                out = res.method.message_count
-        return out
+        return 0
 
     @property
     def is_confirmed_send(self):
@@ -397,8 +386,12 @@ class RMQComm(CommBase.CommBase):
         if routing_key is None:
             routing_key = self.queue
         kwargs.setdefault('mandatory', True)
-        out = self.channel.basic_publish(exchange, routing_key, msg, **kwargs)
-        return out
+        with self.rmq_lock:
+            try:
+                self.channel.basic_publish(exchange, routing_key, msg, **kwargs)
+            except pika.exceptions.UnroutableError:  # pragma: debug
+                return False
+        return True
 
     def _recv(self):
         r"""Receive a message.
@@ -408,17 +401,19 @@ class RMQComm(CommBase.CommBase):
                 message.
 
         """
-        method_frame, props, msg = self.channel.basic_get(
-            queue=self.queue, no_ack=False)
-        if method_frame:
-            self.channel.basic_ack(method_frame.delivery_tag)
-        else:  # pragma: debug
-            raise NoMessages("No messages in connection.")
+        with self.rmq_lock:
+            method_frame, props, msg = self.channel.basic_get(
+                queue=self.queue, auto_ack=False)
+            if method_frame:
+                self.channel.basic_ack(method_frame.delivery_tag)
+            else:  # pragma: debug
+                raise NoMessages("No messages in connection.")
         return (True, msg)
 
     def purge(self):
         r"""Remove all messages from the associated queue."""
-        with self._closing_thread.lock:
-            if self.channel:
-                self.channel.queue_purge(queue=self.queue)
+        with self.rmq_lock:
+            with self._closing_thread.lock:
+                if self.is_open:
+                    self.channel.queue_purge(queue=self.queue)
         super(RMQComm, self).purge()
