@@ -261,6 +261,7 @@ class MSVCCompiler(CCompilerBase):
     # of complex types. Use '/TC' instead of '/TP' for strictly C
     default_flags = ['/W4',      # Display all errors
                      '/Zi',      # Symbolic debug in .pdb (implies debug)
+                     '/FS',      # Allow synchronous write to .pdb
                      # '/MD',
                      # '/MTd',     # Use LIBCMTD.lib to create multithreaded .exe
                      # '/Z7',      # Symbolic debug in .obj (implies debug)
@@ -599,6 +600,17 @@ class CModelDriver(CompiledModelDriver):
         'function': 'python_function_t',
         'instance': 'python_instance_t',
         'any': 'generic_t'}
+    interface_map = {
+        'import': '#include "YggInterface.h"',
+        'input': 'yggInputType("{channel_name}", {datatype})',
+        'output': 'yggOutputType("{channel_name}", {datatype})',
+        'server': 'yggRpcServerType("{channel_name}", {datatype_in}, {datatype_out})',
+        'client': 'yggRpcClientType("{channel_name}", {datatype_out}, {datatype_in})',
+        'timesync': 'yggTimesync("{channel_name}", "{time_units}")',
+        'send': 'flag = yggSend({channel_obj}, {outputs})',
+        'recv': 'flag = yggRecv({channel_obj}, {input_refs})',
+        'call': 'flag = rpcCall({channel_obj}, {outputs}, {inputs})',
+    }
     type_class_map = {}
     function_param = {
         'import': '#include \"{filename}\"',
@@ -1050,31 +1062,18 @@ class CModelDriver(CompiledModelDriver):
         return out
         
     @classmethod
-    def update_io_from_function(cls, model_file, model_function,
-                                inputs=[], outputs=[], **kwargs):
-        r"""Update inputs/outputs from the function definition.
+    def finalize_function_io(cls, direction, x):
+        r"""Finalize info for an input/output channel following function
+        parsing.
 
         Args:
-            model_file (str): Full path to the file containing the model
-                function's declaration.
-            model_function (str): Name of the model function.
-            inputs (list, optional): List of model inputs including types.
-                Defaults to [].
-            outputs (list, optional): List of model outputs including types.
-                Defaults to [].
-            **kwargs: Additional keyword arguments are passed to the parent
-                class's method.
-
-        Returns:
-            dict, None: Flag variable used by the model. If None, the
-                model does not use a flag variable.
+            direction (str): Direction of channel ('input' or 'output')
+            x (dict): Channel info.
 
         """
-        flag_var = super(CModelDriver, cls).update_io_from_function(
-            model_file, model_function, inputs=inputs,
-            outputs=outputs, **kwargs)
-        # Add length_vars if missing for use by yggdrasil
-        for x in inputs:
+        super(CModelDriver, cls).finalize_function_io(direction, x)
+        if direction == 'input':
+            # Add length_vars if missing for use by yggdrasil
             for v in x['vars']:
                 if cls.requires_length_var(v) and (not v.get('length_var', False)):
                     v['length_var'] = {'name': v['name'] + '_length',
@@ -1101,21 +1100,7 @@ class CModelDriver(CompiledModelDriver):
                     #                      'precision': 64},
                     #         'is_length_var': True,
                     #         'dependent': True}
-        for x in outputs:
-            for v in x['vars']:
-                if cls.requires_length_var(v) and (not v.get('length_var', False)):
-                    if v['datatype']['type'] in ['1darray', 'ndarray']:  # pragma: debug
-                        raise RuntimeError("Length must be defined for arrays.")
-                    elif v['datatype'].get('subtype', v['datatype']['type']) == 'bytes':
-                        v['length_var'] = 'strlen(%s)' % v['name']
-                    else:
-                        v['length_var'] = 'strlen4(%s)' % v['name']
-                elif (cls.requires_shape_var(v)
-                      and not (v.get('ndim_var', False)
-                               and v.get('shape_var', False))):  # pragma: debug
-                    raise RuntimeError("Shape must be defined for ND arrays.")
-        # Flag input variables for reallocation
-        for x in inputs:
+            # Flag input variables for reallocation
             allows_realloc = [cls.allows_realloc(v) for v in x['vars']]
             if all(allows_realloc):
                 for v in x['vars']:
@@ -1128,17 +1113,31 @@ class CModelDriver(CompiledModelDriver):
                          and (cls.function_param['recv_function']
                               == cls.function_param['recv_heap']))):
                         v['allow_realloc'] = True
-        for x in inputs + outputs:
-            if x['datatype']['type'] == 'array':
-                nvars_items = len(x['datatype'].get('items', []))
-                nvars = sum([(not ix.get('is_length_var', False))
-                             for ix in x['vars']])
-                if nvars_items == nvars:
-                    x['use_generic'] = False
-                else:
-                    x['use_generic'] = True
-        return flag_var
-        
+        elif direction == 'output':
+            # Add length_vars if missing for use by yggdrasil
+            for v in x['vars']:
+                if cls.requires_length_var(v) and (not v.get('length_var', False)):
+                    if v['datatype']['type'] in ['1darray', 'ndarray']:
+                        if 'iter_datatype' not in v:  # pragma: debug
+                            raise RuntimeError("Length must be defined for "
+                                               "arrays.")
+                    elif v['datatype'].get('subtype', v['datatype']['type']) == 'bytes':
+                        v['length_var'] = 'strlen(%s)' % v['name']
+                    else:
+                        v['length_var'] = 'strlen4(%s)' % v['name']
+                elif (cls.requires_shape_var(v)
+                      and not (v.get('ndim_var', False)
+                               and v.get('shape_var', False))):  # pragma: debug
+                    raise RuntimeError("Shape must be defined for ND arrays.")
+        if x['datatype']['type'] == 'array':
+            nvars_items = len(x['datatype'].get('items', []))
+            nvars = sum([(not ix.get('is_length_var', False))
+                         for ix in x['vars']])
+            if nvars_items == nvars:
+                x['use_generic'] = False
+            else:
+                x['use_generic'] = True
+
     @classmethod
     def input2output(cls, var):
         r"""Perform conversion necessary to turn a variable extracted from a
@@ -1270,7 +1269,7 @@ class CModelDriver(CompiledModelDriver):
         if not ((out == '*') or ('X' in out) or (out == 'double')):
             return out
         from yggdrasil.metaschema.datatypes import get_type_class
-        json_type = kwargs.get('datatype', kwargs.get('type', 'bytes'))
+        json_type = kwargs.get('datatype', kwargs)
         if isinstance(json_type, str):
             json_type = {'type': json_type}
         # if 'type' in kwargs:
@@ -1444,6 +1443,30 @@ class CModelDriver(CompiledModelDriver):
                 for_yggdrasil=True)
         return super(CModelDriver, cls).write_model_recv(channel, recv_var_str, **kwargs)
             
+    @classmethod
+    def write_initialize_oiter(cls, var, value=None, **kwargs):
+        r"""Get the lines necessary to initialize an array for iteration
+        output.
+
+        Args:
+            var (dict, str): Name or information dictionary for the variable
+                being initialized.
+            value (str, optional): Value that should be assigned to the
+                variable.
+            **kwargs: Additional keyword arguments are passed to the
+                parent class's method.
+
+        Returns:
+            list: The lines initializing the variable.
+
+        """
+        value = '({type}*)realloc({name}, {length}*sizeof({type}))'.format(
+            type=cls.get_native_type(**var['iter_datatype']),
+            name=var['name'], length=var['iter_var']['end'])
+        out = super(CModelDriver, cls).write_initialize_oiter(
+            var, value=value, **kwargs)
+        return out
+    
     @classmethod
     def write_declaration(cls, var, **kwargs):
         r"""Return the lines required to declare a variable with a certain
