@@ -22,15 +22,39 @@ class DuplicatedModelDriver(Driver):
 
     name_format = "%s_copy%d"
 
-    def __init__(self, yml, **kwargs):
+    def __init__(self, yml, duplicates=None, **kwargs):
         kwargs.update(yml)
         self.copies = []
-        for iyml in self.get_yaml_copies(yml):
-            ikws = copy.deepcopy(kwargs)
-            ikws.update(iyml)
-            self.copies.append(create_driver(yml=iyml, **ikws))
+        if duplicates is not None:
+            for x in duplicates:
+                ienv = copy.deepcopy(yml.get('env', {}))
+                ienv.update(yml.pop('env_%s' % x['name'], {}))
+                ienv.update(x.pop('env', {}))
+                x['env'] = ienv
+                ikws = copy.deepcopy(kwargs)
+                ikws.update(x)
+                self.copies.append(create_driver(yml=x, **ikws))
+        else:
+            for iyml in self.get_yaml_copies(yml):
+                ikws = copy.deepcopy(kwargs)
+                ikws.update(iyml)
+                self.copies.append(create_driver(yml=iyml, **ikws))
         super(DuplicatedModelDriver, self).__init__(**kwargs)
 
+    @classmethod
+    def get_base_name(cls, name):
+        r"""Get the name of the base model.
+
+        Args:
+            name (str): Model name.
+
+        Returns:
+            str: Base model name.
+
+        """
+        assert('_copy' in name)
+        return name.split('_copy')[0]
+
     @classmethod
     def get_yaml_copies(cls, yml):
         r"""Get a list of yamls for creating duplicate models for the model
@@ -52,6 +76,8 @@ class DuplicatedModelDriver(Driver):
             iyml = copy.deepcopy(yml)
             iyml['name'] = cls.name_format % (yml['name'], i)
             iyml['copy_index'] = i
+            iyml['input_drivers'] = yml['input_drivers']
+            iyml['output_drivers'] = yml['output_drivers']
             # Update environment to reflect addition of suffix
             iyml['env'] = yml.get('env', {}).copy()
             iyml['env'].update(env_copy_specific.get(iyml['name'], {}))
@@ -67,7 +93,11 @@ class DuplicatedModelDriver(Driver):
     def start(self, *args, **kwargs):
         r"""Start thread/process and print info."""
         # self.delay_start(*args, **kwargs)
+        input_drivers = self.yml.get('input_drivers', [])
+        output_drivers = self.yml.get('output_drivers', [])
         for x in self.copies:
+            x.env.update(x.get_io_env(input_drivers=input_drivers,
+                                      output_drivers=output_drivers))
             x.start(*args, **kwargs)
         super(DuplicatedModelDriver, self).start(*args, **kwargs)
 
