@@ -203,9 +203,27 @@ class FortranModelDriver(CompiledModelDriver):
         'function': 'yggpyfunc',
         'instance': 'yggpyinst',
         'any': 'ygggeneric'}
+    interface_map = {
+        'import': 'use fygg',
+        'input': 'ygg_input_type("{channel_name}", {datatype})',
+        'output': 'ygg_output_type("{channel_name}", {datatype})',
+        'server': (
+            'ygg_rpc_server_type("{channel_name}", {datatype_in}, '
+            '{datatype_out})'),
+        'client': (
+            'ygg_rpc_client_type("{channel_name}", {datatype_out}, '
+            '{datatype_in})'),
+        'timesync': 'yggTimesync("{channel_name}", "{time_units}")',
+        'send': 'flag = ygg_send_var({channel_obj}, [yggarg({outputs})])',
+        'recv': 'flag = ygg_recv_var({channel_obj}, [yggarg({inputs})])',
+        'call': (
+            'flag = ygg_rpc_call({channel_obj}, [yggarg({outputs})], '
+            '[yggarg({inputs})])'),
+    }
     function_param = {
         'import_nofile': 'use {function}',
         'import': '#include "{filename}"',
+        'len': 'size({variable},1)',
         'index': '{variable}({index})',
         'interface': 'use fygg',
         'input': ('{channel} = ygg_input_type('
@@ -389,7 +407,7 @@ class FortranModelDriver(CompiledModelDriver):
                     # GNU takes precedence when present
                     cxx_lib = 'stdc++'
                     add_cxx_lib = cxx_lib
-            if add_cxx_lib not in cls.external_libraries:
+            if add_cxx_lib and (add_cxx_lib not in cls.external_libraries):
                 cls.external_libraries[add_cxx_lib] = copy.deepcopy(cxx_orig)
                 cls.internal_libraries['fygg']['external_dependencies'].append(
                     add_cxx_lib)
@@ -821,7 +839,8 @@ class FortranModelDriver(CompiledModelDriver):
                     if match_module:
                         module = match_module.groupdict()['module']
                     else:
-                        module = '%s_module' % kws['function']
+                        module = '%s_module' % kwargs.get('model_name',
+                                                          kws['function'])
                         kws['module'] = module
                     lines.insert(last_use + 1, 'use %s' % module)
                     last_use += 1
@@ -933,6 +952,31 @@ class FortranModelDriver(CompiledModelDriver):
                     return True
         return False
         
+    @classmethod
+    def write_initialize_oiter(cls, var, value=None, **kwargs):
+        r"""Get the lines necessary to initialize an array for iteration
+        output.
+
+        Args:
+            var (dict, str): Name or information dictionary for the variable
+                being initialized.
+            value (str, optional): Value that should be assigned to the
+                variable.
+            **kwargs: Additional keyword arguments are passed to the
+                parent class's method.
+
+        Returns:
+            list: The lines initializing the variable.
+
+        """
+        out = [
+            'if (associated({name}%x)) then'.format(name=var['name']),
+            '   deallocate({name}%x)'.format(name=var['name']),
+            'end if',
+            'allocate({name}%x({size}))'.format(name=var['name'],
+                                                size=var['iter_var']['end'])]
+        return out
+    
     @classmethod
     def write_declaration(cls, var, **kwargs):
         r"""Return the lines required to declare a variable with a certain
@@ -1119,3 +1163,39 @@ class FortranModelDriver(CompiledModelDriver):
         out = super(FortranModelDriver, cls).write_type_decl(
             name, datatype, **kwargs)
         return out
+    
+    @classmethod
+    def format_function_param_len(cls, extra=None, **kwargs):
+        r"""Return the formatted version of the len key.
+
+        Args:
+            extra (dict, optional): Variable dictionary specifying the
+                variable name, datatype, etc. Defaults to None and is ignored.
+            **kwargs: Additional keyword arguments are used in formatting the
+                request function parameter.
+
+        Returns:
+            str: Formatted string.
+
+        """
+        if (extra is not None) and cls.allows_realloc(extra):
+            kwargs['variable'] = "%s%%x" % extra['name']
+        return cls.format_function_param('len', ignore_method=True, **kwargs)
+
+    @classmethod
+    def format_function_param_index(cls, extra=None, **kwargs):
+        r"""Return the formatted version of the index key.
+
+        Args:
+            extra (dict, optional): Variable dictionary specifying the
+                variable name, datatype, etc. Defaults to None and is ignored.
+            **kwargs: Additional keyword arguments are used in formatting the
+                request function parameter.
+
+        Returns:
+            str: Formatted string.
+
+        """
+        if (extra is not None) and cls.allows_realloc(extra):
+            kwargs['variable'] = "%s%%x" % extra['name']
+        return cls.format_function_param('index', ignore_method=True, **kwargs)
