@@ -97,7 +97,7 @@ void set_sent_eof(const comm_t* x) {
 
 /*!
   @brief Retrieve a registered global comm if it exists.
-  @param[in] const char* name Name that comm might be registered under.
+  @param[in] name const char* name Name that comm might be registered under.
   @returns comm_t* Pointer to registered comm. NULL if one does not exist
   with the specified name.
  */
@@ -158,7 +158,7 @@ int is_comm_format_array_type(const comm_t *x) {
 
 /*!
   @brief Determine if the current thread can use a comm registered by another.
-  @param[in] int Thread that created the comm.
+  @param[in] thread_id int Thread that created the comm.
   @returns int 1 if the current thread can use the comm, 0 otherwise.
  */
 static
@@ -662,6 +662,11 @@ comm_head_t comm_send_multipart_header(const comm_t *x, const char * data,
   if (model_name != NULL) {
     strcpy(head.model, model_name);
   }
+  char *model_copy = getenv("YGG_MODEL_COPY");
+  if (model_copy != NULL) {
+    strcat(head.model, "_copy");
+    strcat(head.model, model_copy);
+  }
   head.flags = head.flags | HEAD_FLAG_VALID | HEAD_FLAG_MULTIPART;
   // Add datatype information to header
   if (!(x->flags & COMM_FLAG_FILE)) {
@@ -1024,13 +1029,18 @@ int comm_recv_multipart(comm_t *x, char **data, const size_t len,
       return -2;
     }
     // Get datatype information from header on first recv
-    dtype_t *updtype;
+    dtype_t *updtype = NULL;
     if (x->type == SERVER_COMM) {
       comm_t *handle = (comm_t*)(x->handle);
       updtype = handle->datatype;
     } else {
       updtype = x->datatype;
     }
+    if (updtype == NULL) {
+      ygglog_error("comm_recv_multipart(%s): Datatype is NULL.", x->name);
+      destroy_header(&head);
+      return -1;
+    }
     if ((!(x->const_flags[0] & COMM_FLAGS_USED)) && (!(x->flags & COMM_FLAG_FILE)) && (updtype->obj == NULL) && (!(head.flags & HEAD_TYPE_IN_DATA))) {
       ygglog_debug("comm_recv_multipart(%s): Updating datatype to '%s'",
 		   x->name, head.dtype->type);
@@ -1041,6 +1051,8 @@ int comm_recv_multipart(comm_t *x, char **data, const size_t len,
 	return -1;
       }
     } else if ((!(x->flags & COMM_FLAG_FILE)) && (head.dtype != NULL)) {
+      ygglog_debug("comm_recv_multipart(%s): Updating existing datatype to '%s' from '%s'",
+		   x->name, head.dtype->type, updtype->type);
       ret = update_dtype(updtype, head.dtype);
       if (ret != 0) {
 	ygglog_error("comm_recv_multipart(%s): Error updating existing datatype.", x->name);
@@ -1049,6 +1061,7 @@ int comm_recv_multipart(comm_t *x, char **data, const size_t len,
       }
     }
     if (head.flags & HEAD_FLAG_MULTIPART) {
+      ygglog_debug("comm_recv_multipart(%s): Message is multipart", x->name);
       // Return early if header contained entire message
       if (head.size == head.bodysiz) {
         x->const_flags[0] = x->const_flags[0] | COMM_FLAGS_USED;
@@ -1132,6 +1145,7 @@ int comm_recv_multipart(comm_t *x, char **data, const size_t len,
       }
       free_comm(xmulti);
     } else {
+      ygglog_debug("comm_recv_multipart(%s): Message not multipart", x->name);
       ret = (int)(head.bodysiz);
     }
   }
