@@ -54,6 +54,8 @@ public:
   /*!
     @brief Copy data wrapped in YggGeneric class.
     @param[in] data YggGeneric* Pointer to generic object.
+    @param[in] orig_data Pointer to data that should be copied if different
+    that the data that is wrapped.
     @returns void* Pointer to copy of data.
    */
   void* copy_generic(const YggGeneric* data, void* orig_data=NULL) const override {
@@ -139,6 +141,15 @@ public:
   size_t nargs_exp() const override {
     return 1;
   }
+  /*!
+    @brief Skip arguments that make of this type.
+    @param[in, out] nargs Pointer to number of arguments in ap.
+    @param[in, out] ap va_list_t Variable argument list.
+   */
+  void skip_va_elements_core(size_t *nargs, va_list_t *ap) const override {
+    va_arg(ap->va, ply_t);
+    (*nargs)--;
+  }
   /*!
     @brief Convert a Python representation to a C representation.
     @param[in] pyobj PyObject* Pointer to Python object.
