@@ -1,3 +1,4 @@
+import os
 from yggdrasil import tools
 from yggdrasil.communication.DefaultComm import DefaultComm
 
@@ -5,6 +6,10 @@ from yggdrasil.communication.DefaultComm import DefaultComm
 YGG_MSG_MAX = tools.get_YGG_MSG_MAX()
 YGG_MSG_EOF = tools.YGG_MSG_EOF
 YGG_MSG_BUF = tools.YGG_MSG_BUF
+YGG_SERVER_INPUT = os.environ.get('YGG_SERVER_INPUT', False)
+YGG_SERVER_OUTPUT = os.environ.get('YGG_SERVER_OUTPUT', False)
+YGG_MODEL_NAME = os.environ.get('YGG_MODEL_NAME', False)
+_global_scope_comms = {}
 
 
 def maxMsgSize():
@@ -22,13 +27,15 @@ def eof_msg():
     return YGG_MSG_EOF
 
 
-def YggInit(_type, args=None):
+def YggInit(_type, args=None, kwargs=None):
     r"""Short interface to identify functions called by another language.
 
     Args:
         _type (str): Name of class that should be returned.
-        args (list): Additional arguments that should be passed to class
-            initializer.
+        args (list, optional): Additional arguments that should be passed to
+            the class initializer.
+        kwargs (dict, optional): Additional keyword arguments that should be
+            passed to the class initializer.
 
     Returns:
         obj: An instance of the requested class.
@@ -36,6 +43,8 @@ def YggInit(_type, args=None):
     """
     if args is None:
         args = []
+    if kwargs is None:
+        kwargs = {}
     if _type.startswith('Psi'):
         _type = _type.replace('Psi', 'Ygg', 1)
     elif _type.startswith('PSI'):
@@ -48,26 +57,35 @@ def YggInit(_type, args=None):
     if isinstance(cls, (int, bytes, str)):
         obj = cls
     else:
-        obj = cls(*args)
+        obj = cls(*args, **kwargs)
     return obj
 
 
-def InterfaceComm(name, comm_class=None, **kwargs):
+def InterfaceComm(name, comm_class=None, global_scope=False, **kwargs):
     r"""Short hand for initializing a default comm for use as an interface.
 
     Args:
         name (str): The name of the message queue.
         comm_class (CommBase, optional): Communication class that should be
             used. Defaults to DefaultComm if not provided.
+        global_scope (bool, optional): If True, the comm will be checked
+            for in the global scope before creating a new one. Defaults to
+            False.
         **kwargs: Additional keyword arguments are passed to DefaultComm.
 
     """
+    global _global_scope_comms
+    if global_scope and (name in _global_scope_comms):
+        return _global_scope_comms[name]
     if comm_class is None:
         comm_class = DefaultComm
     kwargs.update(is_interface=True)
     if 'language' not in kwargs:
         kwargs['language'] = tools.get_subprocess_language()
-    return comm_class(name, **kwargs)
+    out = comm_class(name, **kwargs)
+    if global_scope:
+        _global_scope_comms[name] = out
+    return out
 
 
 def YggInput(name, format_str=None, **kwargs):
@@ -86,6 +104,11 @@ def YggInput(name, format_str=None, **kwargs):
         DefaultComm: Communication object.
         
     """
+    if ((YGG_SERVER_INPUT
+         and ((name == YGG_SERVER_INPUT)
+              or (('%s:%s' % (YGG_MODEL_NAME, name)) == YGG_SERVER_INPUT)))):
+        return YggRpcServer(YGG_MODEL_NAME, infmt=format_str,
+                            outfmt=None, global_scope=True)
     if format_str is not None:
         kwargs['format_str'] = format_str
     kwargs.update(direction='recv', recv_timeout=False)
@@ -108,13 +131,18 @@ def YggOutput(name, format_str=None, **kwargs):
         DefaultComm: Communication object.
         
     """
+    if ((YGG_SERVER_OUTPUT
+         and ((name == YGG_SERVER_OUTPUT)
+              or (('%s:%s' % (YGG_MODEL_NAME, name)) == YGG_SERVER_OUTPUT)))):
+        return YggRpcServer(YGG_MODEL_NAME, outfmt=format_str,
+                            infmt=None, global_scope=True)
     if format_str is not None:
         kwargs['format_str'] = format_str
     kwargs.update(direction='send')
     return InterfaceComm(name, **kwargs)
 
     
-def YggRpcServer(name, infmt='%s', outfmt='%s'):
+def YggRpcServer(name, infmt=None, outfmt=None, **kwargs):
     r"""Get class for handling requests and response for an RPC Server.
 
     Args:
@@ -123,21 +151,26 @@ def YggRpcServer(name, infmt='%s', outfmt='%s'):
             messages received from the request queue. Defaults to '%s'.
         outfmt (str, optional): Format string used to format variables in a
             message sent to the response queue. Defautls to '%s'.
+        **kwargs: Additional keyword arguments are passed to InterfaceComm.
 
     Returns:
         :class:.ServerComm: Communication object.
         
     """
     from yggdrasil.communication import ServerComm
-    icomm_kwargs = dict(format_str=infmt)
-    ocomm_kwargs = dict(format_str=outfmt)
-    out = InterfaceComm(name, comm_class=ServerComm.ServerComm,
-                        response_kwargs=ocomm_kwargs,
-                        recv_timeout=False, **icomm_kwargs)
-    return out
+    icomm_kwargs = {}
+    ocomm_kwargs = {}
+    if infmt is not None:
+        icomm_kwargs['format_str'] = infmt
+    if outfmt is not None:
+        ocomm_kwargs['format_str'] = outfmt
+    kwargs.update(icomm_kwargs, comm_class=ServerComm.ServerComm,
+                  response_kwargs=ocomm_kwargs)
+    kwargs.setdefault('recv_timeout', False)
+    return InterfaceComm(name, **kwargs)
 
 
-def YggRpcClient(name, outfmt='%s', infmt='%s'):
+def YggRpcClient(name, outfmt=None, infmt=None, **kwargs):
     r"""Get class for handling requests and response to an RPC Server from a
     client.
 
@@ -147,35 +180,60 @@ def YggRpcClient(name, outfmt='%s', infmt='%s'):
             message sent to the request queue. Defautls to '%s'.
         infmt (str, optional): Format string used to recover variables from
             messages received from the response queue. Defautls to '%s'.
+        **kwargs: Additional keyword arguments are passed to InterfaceComm.
 
     Returns:
         :class:.ClientComm: Communication object.
         
     """
     from yggdrasil.communication import ClientComm
-    icomm_kwargs = dict(format_str=infmt)
-    ocomm_kwargs = dict(format_str=outfmt)
-    out = InterfaceComm(name, comm_class=ClientComm.ClientComm,
-                        response_kwargs=icomm_kwargs,
-                        recv_timeout=False, **ocomm_kwargs)
-    return out
+    icomm_kwargs = {}
+    ocomm_kwargs = {}
+    if infmt is not None:
+        icomm_kwargs['format_str'] = infmt
+    if outfmt is not None:
+        ocomm_kwargs['format_str'] = outfmt
+    kwargs.update(ocomm_kwargs, comm_class=ClientComm.ClientComm,
+                  response_kwargs=icomm_kwargs)
+    kwargs.setdefault('recv_timeout', False)
+    return InterfaceComm(name, **kwargs)
+
+
+def YggTimesyncServer(name='timesync', **kwargs):
+    r"""Get class for handling timestep synchronization requests on
+    the driver side.
+
+    Args:
+        name (str, optional): The name of the server queues. Defaults
+            to 'timesync'.
+        **kwargs: Additional keyword arguments are passed to InterfaceComm.
+
+    Returns:
+        :class:.ServerComm: Communication object.
+        
+    """
+    from yggdrasil.communication import ServerComm
+    kwargs['comm_class'] = ServerComm.ServerComm
+    kwargs.setdefault('recv_timeout', False)
+    return InterfaceComm(name, **kwargs)
     
 
-def YggTimesync(name='timesync'):
+def YggTimesync(name='timesync', **kwargs):
     r"""Get class for handling timestep synchronization requests.
 
     Args:
         name (str, optional): The name of the server queues. Defaults
             to 'timesync'.
+        **kwargs: Additional keyword arguments are passed to InterfaceComm.
 
     Returns:
         :class:.ClientComm: Communication object.
         
     """
     from yggdrasil.communication import ClientComm
-    out = InterfaceComm(name, comm_class=ClientComm.ClientComm,
-                        recv_timeout=False)
-    return out
+    kwargs['comm_class'] = ClientComm.ClientComm
+    kwargs.setdefault('recv_timeout', False)
+    return InterfaceComm(name, **kwargs)
 
 
 # Specialized classes for ascii IO
@@ -267,7 +325,7 @@ def YggAsciiArrayInput(name, **kwargs):
     return YggAsciiTableInput(name, **kwargs)
 
 
-def YggAsciiArrayOutput(name, fmt, **kwargs):
+def YggAsciiArrayOutput(name, fmt=None, **kwargs):
     r"""Get class for handling table-like formatted output as arrays.
 
     Args:
