@@ -1,5 +1,12 @@
+  ! BEGIN DOXYGEN_SHOULD_SKIP_THIS
   interface
      ! Utilities
+     function ygg_init_c() result(out) bind(c, name="ygg_init_f")
+       use, intrinsic :: iso_c_binding, only: c_int
+       implicit none
+       integer(kind=c_int) :: out
+     end function ygg_init_c
+     
      subroutine c_free(x) bind(c, name="ygg_c_free")
        use, intrinsic :: iso_c_binding, only: c_ptr
        implicit none
@@ -25,6 +32,13 @@
        use, intrinsic :: iso_c_binding, only: c_int
        integer(kind=c_int), intent(in), value :: seconds
      end subroutine fsleep
+
+     subroutine set_global_comm() bind(c, name="set_global_comm_f")
+       implicit none
+     end subroutine set_global_comm
+     subroutine unset_global_comm() bind(c, name="unset_global_comm_f")
+       implicit none
+     end subroutine unset_global_comm
   
      ! Methods for initializing channels
      function is_comm_format_array_type_c(x) result(out) &
@@ -253,6 +267,26 @@
        type(c_ptr) :: channel
      end function ygg_rpc_server_c
 
+     function ygg_rpc_client_type_c(name, out_type, in_type) result(channel) &
+          bind(c, name="yggRpcClientType_f")
+       use, intrinsic :: iso_c_binding, only: c_ptr, c_char
+       implicit none
+       character(kind=c_char), dimension(*), intent(in) :: name
+       type(c_ptr), value, intent(in) :: out_type
+       type(c_ptr), value, intent(in) :: in_type
+       type(c_ptr) :: channel
+     end function ygg_rpc_client_type_c
+
+     function ygg_rpc_server_type_c(name, in_type, out_type) result(channel) &
+          bind(c, name="yggRpcServerType_f")
+       use, intrinsic :: iso_c_binding, only: c_ptr, c_char
+       implicit none
+       character(kind=c_char), dimension(*), intent(in) :: name
+       type(c_ptr), value, intent(in) :: in_type
+       type(c_ptr), value, intent(in) :: out_type
+       type(c_ptr) :: channel
+     end function ygg_rpc_server_type_c
+
      function ygg_timesync_c(name, t_units) result(channel) &
           bind(c, name="yggTimesync_f")
        use, intrinsic :: iso_c_binding, only: c_ptr, c_char
@@ -948,3 +982,4 @@
      end function generic_map_set_ndarray_c
 
   end interface
+#endif ! DOXYGEN_SHOULD_SKIP_THIS
