@@ -7,9 +7,12 @@
 extern "C" {
 #endif
 
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
+
 const int YGG_MSG_MAX_F = YGG_MSG_MAX;
 
 // Utilities
+int ygg_init_f();
 void ygg_c_free(void *x);
 void ygg_log_info_f(const char* fmt);
 void ygg_log_debug_f(const char* fmt);
@@ -176,9 +179,10 @@ int generic_map_set_ndarray_f(generic_t x, const char* key,
 			      const size_t ndim, const void* shape,
 			      const char* units);
 
+#endif // DOXYGEN_SHOULD_SKIP_THIS
+
 #ifdef __cplusplus /* If this is a C++ compiler, end C linkage */
 }
 #endif
 
-
 #endif /*YGG_FC_WRAPPERS_H_*/
